package Viewer;

use strict;
use warnings;

use Gtk2 qw( -init );
use Gtk2::Builder::Simple;
use IO::File;

use base qw( Gtk2::Builder::Simple );

sub new {
	my $class = shift;
	my $self = $class->SUPER::new( 'viewer.ui' );
	return $self;
}

sub on_open_activate {
	my $self = shift;
	my $dlg = Gtk2::FileChooserDialog->new(
		'Please choose file to open',
		$self->{window},
		'open',
		_Cancel => 'cancel',
		_Open => 'ok',
	);

	if( 'ok' eq $dlg->run ) {
		# Hide dialog in case there is an error
		$dlg->hide;
		$self->load_file( $dlg->get_filename );
	}

	$dlg->destroy;
}

sub load_file {
	my $self = shift;
	my ($file) = @_;
	my $buffer = $self->{textview}->get_buffer;

	$buffer->set_text( '' );

	eval {
		my $fh = IO::File->new;
		my $iter = $buffer->get_start_iter;

		$fh->open( $file, '<:utf8' ) or die( $! );

		undef( $! );
		for( my $line;; ) {
			unless( defined( $line = $fh->getline ) ) {
				if( $! ) {
					$buffer->set_text( '' );
					die( $! );
				}

				last;
			}

			$buffer->insert( $iter, $line );
		}

		$fh->close;
	};
	if( $@ ) {
		my $dialog = Gtk2::MessageDialog->new(
			$self->{window},
			[ qw( modal destroy-with-parent ) ],
			'error',
			'ok',
			'Failed to read the file.'
		);

		$dialog->run;
		$dialog->destroy;
	}
}

sub gtk_main_quit {
	Gtk2->main_quit;
}


1;

package main;

Viewer->new->run;


1;
